#!/usr/bin/env python3

# This script is intended to be called automatically every day (e.g. via cron).
# It only output stuff if new ranks have to be inserted. Therefore the output
# may be redirected to email notifying about manual action to transfer the
# ranks to MySQL.
#
# Configure cron as the user running the DDNet-Server processes
#
#     $ crontab -e
#     30 5 * * * /path/to/this/script/move_sqlite.py --from /path/to/ddnet-server.sqlite
#
# Afterwards configure a MTA (e.g. postfix) and the users email address.

import sqlite3
import argparse
from time import strftime
import os

def sqlite_table_exists(cursor, table):
	cursor.execute("SELECT COUNT(*) FROM sqlite_master WHERE type='table' AND name='{}'".format(table))
	return cursor.fetchone()[0] != 0

def sqlite_num_transfer(conn, table):
	c = conn.cursor()
	if not sqlite_table_exists(c, table):
		return 0
	c.execute('SELECT COUNT(*) FROM {}'.format(table))
	num = c.fetchone()[0]
	return num

def transfer(file_from, file_to):
	conn_to = sqlite3.connect(file_to, isolation_level='EXCLUSIVE')
	cursor_to = conn_to.cursor()

	conn_from = sqlite3.connect(file_from, isolation_level='EXCLUSIVE')
	for line in conn_from.iterdump():
		cursor_to.execute(line)
		print(line)
	cursor_to.close()
	conn_to.commit()
	conn_to.close()

	cursor_from = conn_from.cursor()
	if sqlite_table_exists(cursor_from, 'record_race'):
		cursor_from.execute('DELETE FROM record_race')
	if sqlite_table_exists(cursor_from, 'record_teamrace'):
		cursor_from.execute('DELETE FROM record_teamrace')
	if sqlite_table_exists(cursor_from, 'record_saves'):
		cursor_from.execute('DELETE FROM record_saves')
	cursor_from.close()
	conn_from.commit()
	conn_from.close()

def main():
	default_output = 'ddnet-server-' + strftime('%Y-%m-%d') + '.sqlite'
	parser = argparse.ArgumentParser(
			description='Move DDNet ranks, teamranks and saves from a possible active SQLite3 to a new one',
			formatter_class=argparse.ArgumentDefaultsHelpFormatter)
	parser.add_argument('--from', '-f', dest='f',
			default='ddnet-server.sqlite',
			help='Input file where ranks are deleted from when moved successfully (default: ddnet-server.sqlite)')
	parser.add_argument('--to', '-t',
			default=default_output,
			help='Output file where ranks are saved adds current date by default')
	args = parser.parse_args()

	if not os.path.exists(args.f):
		print("Warning: '{}' does not exist (yet). Is the path specified correctly?".format(args.f))
		return

	conn = sqlite3.connect(args.f)
	num_ranks = sqlite_num_transfer(conn, 'record_race')
	num_teamranks = sqlite_num_transfer(conn, 'record_teamrace')
	num_saves = sqlite_num_transfer(conn, 'record_saves')
	num = num_ranks + num_teamranks + num_saves
	conn.close()
	if num == 0:
		return

	print('{} new entries in backup database found ({} ranks, {} teamranks, {} saves'.format(num, num_ranks, num_teamranks, num_saves))
	print('Moving entries from {} to {}'.format(
			os.path.abspath(args.f),
			os.path.abspath(args.to)))
	sql_file = 'ddnet-server-' + strftime('%Y-%m-%d') + '.sql'
	print("You can use the following commands to import the entries to MySQL (use sed 's/record_/prefix_/' for other database prefixes):")
	print()
	print(("    echo '.dump --preserve-rowids' | sqlite3 {} | " + # including rowids, this forces sqlite to name all columns in each INSERT statement
			"grep -E '^INSERT INTO record_(race|teamrace|saves)' | " + # filter out inserts
			"sed -e 's/INSERT INTO/INSERT IGNORE INTO/' | " + # ignore duplicate rows
			"sed -e 's/rowid,//' -e 's/VALUES([0-9]*,/VALUES(/' > {}") # filter out rowids again
			.format(os.path.abspath(args.to), sql_file))
	print("    mysql -u teeworlds -p'PW2' teeworlds < {}".format(sql_file))
	print()
	print("When the ranks are transfered successfully to mysql, {} can be removed".format(
			os.path.abspath(args.to)))
	print()
	print("Log of the transfer:")
	print()

	transfer(args.f, args.to)

if __name__ == '__main__':
	main()



print("#ifndef GENERATED_WORDLIST_H")
print("#define GENERATED_WORDLIST_H")
print("const char g_aFallbackWordlist[][32] = {")
with open("data/wordlist.txt") as f:
	for line in f:
		word = line.strip().split("\t")[1]
		print("\t\"%s\", " % word)
print("};")
print("#endif // GENERATED_WORDLIST_H")



import os
import re
from collections import OrderedDict

class LanguageDecodeError(Exception):
	def __init__(self, message, filename, line):
		error = "File \"{1}\", line {2}: {0}".format(message, filename, line+1)
		super().__init__(error)


# Taken from https://stackoverflow.com/questions/30011379/how-can-i-parse-a-c-format-string-in-python
cfmt = r'''\
(                                  # start of capture group 1
%                                  # literal "%"
(?:                                # first option
(?:[-+0 #]{0,5})                   # optional flags
(?:\d+|\*)?                        # width
(?:\.(?:\d+|\*))?                  # precision
(?:h|l|ll|w|I|I32|I64)?            # size
[cCdiouxXeEfgGaAnpsSZ]             # type
) |                                # OR
%%)                                # literal "%%"
'''


def decode(fileobj, elements_per_key):
	data = {}
	current_context = ""
	current_key = None
	index = -1
	for index, line in enumerate(fileobj):
		line = line.encode("utf-8").decode("utf-8-sig")
		line = line[:-1]
		if line and line[-1] == "\r":
			line = line[:-1]
		if not line or line[:1] == "#":
			current_context = ""
			continue

		if line[0] == "[":
			if line[-1] != "]":
				raise LanguageDecodeError("Invalid context string", fileobj.name, index)
			current_context = line[1:-1]
		elif line[:3] == "== ":
			if len(data[current_key]) >= 1+elements_per_key:
				raise LanguageDecodeError("Wrong number of elements per key", fileobj.name, index)
			if current_key:
				original = current_key[0] # pylint: disable=E1136
				translation = line[3:]
				if translation and [m.group(1) for m in re.finditer(cfmt, original, flags=re.X)] != [m.group(1) for m in re.finditer(cfmt, translation, flags=re.X)]:
					raise LanguageDecodeError("Non-matching formatting string", fileobj.name, index)
				data[current_key].extend([translation])
			else:
				raise LanguageDecodeError("Element before key given", fileobj.name, index)
		else:
			if current_key:
				if len(data[current_key]) != 1+elements_per_key:
					raise LanguageDecodeError("Wrong number of elements per key", fileobj.name, index)
				data[current_key].append(index)
			if line in data:
				raise LanguageDecodeError("Key defined multiple times: " + line, fileobj.name, index)
			data[(line, current_context)] = [index]
			current_key = (line, current_context)
	if len(data[current_key]) != 1+elements_per_key:
		raise LanguageDecodeError("Wrong number of elements per key", fileobj.name, index)
	data[current_key].append(index+1)
	return data


def check_file(path):
	with open(path) as fileobj:
		matches = re.findall(r"Localize\s*\(\s*\"([^\"]+)\"(?:\s*,\s*\"([^\"]+)\")?\s*\)", fileobj.read())
	return matches


def check_folder(path):
	englishlist = OrderedDict()
	for path2, dirs, files in os.walk(path):
		dirs.sort()
		for f in sorted(files):
			if not any(f.endswith(x) for x in ".cpp .c .h".split()):
				continue
			for sentence in check_file(os.path.join(path2, f)):
				englishlist[sentence] = None
	return englishlist


def languages():
	index = decode(open("data/languages/index.txt"), 2)
	langs = {"data/languages/"+key[0]+".txt" : [key[0]]+elements for key, elements in index.items()}
	return langs


def translations(filename):
	return decode(open(filename), 1)


def localizes():
	englishlist = list(check_folder("src"))
	return englishlist



# Needs UnicodeData.txt in the current directory.
#
# It can be obtained from unicode.org:
# - http://www.unicode.org/Public/<VERSION>/ucd/UnicodeData.txt
#
# If executed as a script, it will generate the contents of the file
# `src/base/unicode/tolower_data.h`.

import unicode

def generate_cases():
	ud = unicode.data()
	return [(unicode.unhex(u["Value"]), unicode.unhex(u["Simple_Lowercase_Mapping"])) for u in ud if u["Simple_Lowercase_Mapping"]]

def main():
	cases = generate_cases()

	print("""\
#include <stdint.h>

struct UPPER_LOWER
{{
\tint32_t upper;
\tint32_t lower;
}};

enum
{{
\tNUM_TOLOWER={},
}};

static const struct UPPER_LOWER tolower[NUM_TOLOWER] = {{""".format(len(cases)))
	for upper_code, lower_code in cases:
		print("\t{{{}, {}}},".format(upper_code, lower_code))
	print("};")

if __name__ == '__main__':
	main()



#!/usr/bin/env python3

import os
import subprocess
import sys
import argparse

os.chdir(os.path.dirname(__file__) + "/..")

def recursive_file_list(path):
	result = []
	for dirpath, _, filenames in os.walk(path):
		result += [os.path.join(dirpath, filename) for filename in filenames]
	return result

IGNORE_FILES = [
	"src/engine/client/keynames.h",
	"src/engine/keys.h",
]
def filter_ignored(filenames):
	return [filename for filename in filenames
		if filename not in IGNORE_FILES
		and not filename.startswith("src/game/generated/")]

def filter_cpp(filenames):
	return [filename for filename in filenames
		if any(filename.endswith(ext) for ext in ".c .cpp .h".split())]

def reformat(filenames):
	subprocess.check_call(["clang-format", "-i"] + filenames)

def warn(filenames):
	return subprocess.call(["clang-format", "-Werror", "--dry-run"] + filenames)

def main():
	p = argparse.ArgumentParser(description="Check and fix style of changed files")
	p.add_argument("-n", "--dry-run", action="store_true", help="Don't fix, only warn")
	args = p.parse_args()
	filenames = filter_ignored(filter_cpp(recursive_file_list("src")))
	if not args.dry_run:
		reformat(filenames)
	else:
		sys.exit(warn(filenames))

if __name__ == "__main__":
	main()



import network
import seven.network

def get_msgs():
	return ["NETMSG_INVALID"] + [m.enum_name for m in network.Messages]

def get_msgs_7():
	return ["NETMSG_INVALID"] + [m.enum_name for m in seven.network.Messages]

def get_objs():
	return ["NETOBJ_INVALID"] + [m.enum_name for m in network.Objects if m.ex is None]

def get_objs_7():
	return ["NETOBJ_INVALID"] + [m.enum_name for m in seven.network.Objects]

def generate_map(a, b):
	result = []
	for m in a:
		try:
			result += [b.index(m)]
		except ValueError:
			result += [-1]

	return result


def output_map(name, m):
	print("static const int gs_{}[{}] = {{".format(name, len(m)))
	print(*m, sep=',')
	print("};")
	print("inline int {0}(int a) {{ if(a < 0 || a >= {1}) return -1; return gs_{0}[a]; }}".format(name, len(m)))

def main():
	guard = "GAME_GENERATED_PROTOCOLGLUE"
	print("#ifndef " + guard)
	print("#define " + guard)

	msgs = get_msgs()
	msgs7 = get_msgs_7()

	output_map("Msg_SixToSeven", generate_map(msgs, msgs7))
	output_map("Msg_SevenToSix", generate_map(msgs7, msgs))

	objs = get_objs()
	objs7 = get_objs_7()
	output_map("Obj_SixToSeven", generate_map(objs, objs7))
	output_map("Obj_SevenToSix", generate_map(objs7, objs))

	print("#endif //" + guard)

if __name__ == "__main__":
	main()
